/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util.btree;


public class CLRSTest {

	public Node root;
	public Node child1;
	public Node child2;
	public Node child3;
	public Node child4;
	public Node child5;

	public void validateInsertB() {
		assert(root.getNumKeys() == 4);
		assert(child1.getNumKeys() == 5);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 5);
		assert(child5.getNumKeys() == 2);
	}

	public void validateInsertQ() {
		child5 = root.getChild(5);
		Node child6 = root.getChild(6);

		assert(root.getNumKeys() == 5);
		assert(child1.getNumKeys() == 5);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 3);
		assert(child5.getNumKeys() == 2);
		assert(child6.getNumKeys() == 2);
	}

	public void validateInsertL(Store store) {

		root = store.get(store.getRootAddress());
		assert(root.getNumKeys() == 1);

		child1 = root.getChild(1);
		child2 = root.getChild(2);

		assert(child1.getNumKeys() == 2);
		assert(child2.getNumKeys() == 2);

		child3 = child1.getChild(1);
		child4 = child1.getChild(2);
		child5 = child1.getChild(3);

		Node child6 = child2.getChild(1);
		Node child7 = child2.getChild(2);
		Node child8 = child2.getChild(3);

		assert(child3.getNumKeys() == 5);
		assert(child4.getNumKeys() == 3);
		assert(child5.getNumKeys() == 2);
		assert(child6.getNumKeys() == 3);
		assert(child7.getNumKeys() == 2);
		assert(child8.getNumKeys() == 2);	
	}
	
	public void validateInsertF() {

		assert(root.getNumKeys() == 1);

		child1 = root.getChild(1);
		child2 = root.getChild(2);

		assert(child1.getNumKeys() == 3);
		assert(child2.getNumKeys() == 2);

		child3 = child1.getChild(1);
		child4 = child1.getChild(2);
		child5 = child1.getChild(3);
		Node child6 = child1.getChild(4);

		Node child7 = child2.getChild(1);
		Node child8 = child2.getChild(2);
		Node child9 = child2.getChild(3);

		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 3);
		assert(child5.getNumKeys() == 3);
		assert(child6.getNumKeys() == 2);
		assert(child7.getNumKeys() == 3);
		assert(child8.getNumKeys() == 2);
		assert(child9.getNumKeys() == 2);
	}

	public void validateDeleteF() {

		assert(root.getNumKeys() == 1);

		child1 = root.getChild(1);
		child2 = root.getChild(2);

		assert(child1.getNumKeys() == 3);
		assert(child2.getNumKeys() == 2);

		child3 = child1.getChild(1);
		child4 = child1.getChild(2);
		child5 = child1.getChild(3);
		Node child6 = child1.getChild(4);

		Node child7 = child2.getChild(1);
		Node child8 = child2.getChild(2);
		Node child9 = child2.getChild(3);

		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 2);
		assert(child5.getNumKeys() == 3);
		assert(child6.getNumKeys() == 2);
		assert(child7.getNumKeys() == 3);
		assert(child8.getNumKeys() == 2);
		assert(child9.getNumKeys() == 2);
		
		assert(child9.getKey(1).getName().equals("Y"));
		assert(child9.getKey(2).getName().equals("Z"));
	}

	public void validateDeleteM() {

		assert(root.getNumKeys() == 1);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);

		assert(child1.getNumKeys() == 3);
		assert(child2.getNumKeys() == 2);
		assert(child1.isLeaf() == false);
		assert(child2.isLeaf() == false);

		child3 = child1.getChild(1);
		child4 = child1.getChild(2);
		child5 = child1.getChild(3);
		Node child6 = child1.getChild(4);

		Node child7 = child2.getChild(1);
		Node child8 = child2.getChild(2);
		Node child9 = child2.getChild(3);

		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 2);
		assert(child5.getNumKeys() == 2);
		assert(child6.getNumKeys() == 2);
		assert(child7.getNumKeys() == 3);
		assert(child8.getNumKeys() == 2);
		assert(child9.getNumKeys() == 2);
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		assert(child5.isLeaf() == true);
		assert(child6.isLeaf() == true);
		assert(child7.isLeaf() == true);
		assert(child8.isLeaf() == true);
		assert(child9.isLeaf() == true);
		
		assert(child9.getKey(1).getName().equals("Y"));
		assert(child9.getKey(2).getName().equals("Z"));		
	}
	
	public void validateDeleteG() {

		assert(root.getNumKeys() == 1);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);

		assert(child1.getNumKeys() == 2);
		assert(child2.getNumKeys() == 2);
		assert(child1.isLeaf() == false);
		assert(child2.isLeaf() == false);

		child3 = child1.getChild(1);
		child4 = child1.getChild(2);
		child5 = child1.getChild(3);

		Node child6 = child2.getChild(1);
		Node child7 = child2.getChild(2);
		Node child8 = child2.getChild(3);

		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 4);
		assert(child5.getNumKeys() == 2);
		assert(child6.getNumKeys() == 3);
		assert(child7.getNumKeys() == 2);
		assert(child8.getNumKeys() == 2);
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		assert(child5.isLeaf() == true);
		assert(child6.isLeaf() == true);
		assert(child7.isLeaf() == true);
		assert(child8.isLeaf() == true);
		
		assert(child8.getKey(1).getName().equals("Y"));
		assert(child8.getKey(2).getName().equals("Z"));		
	}

	public void validateDeleteD(Store store) {
		root = store.get(store.getRootAddress());
		assert(root.getNumKeys() == 5);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);
		child3 = root.getChild(3);
		child4 = root.getChild(4);
		child5 = root.getChild(5);
		Node child6 = root.getChild(6);

		assert(child1.getNumKeys() == 2);
		assert(child2.getNumKeys() == 3);
		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 3);
		assert(child5.getNumKeys() == 2);
		assert(child6.getNumKeys() == 2);

		assert(child1.isLeaf() == true);
		assert(child2.isLeaf() == true);		
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		assert(child5.isLeaf() == true);
		assert(child6.isLeaf() == true);
		
		assert(child6.getKey(1).getName().equals("Y"));
		assert(child6.getKey(2).getName().equals("Z"));		
	}

	public void validateDeleteB() {

		assert(root.getNumKeys() == 5);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);
		child3 = root.getChild(3);
		child4 = root.getChild(4);
		child5 = root.getChild(5);
		Node child6 = root.getChild(6);

		assert(child1.getNumKeys() == 2);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 3);
		assert(child5.getNumKeys() == 2);
		assert(child6.getNumKeys() == 2);

		assert(child1.isLeaf() == true);
		assert(child2.isLeaf() == true);		
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		assert(child5.isLeaf() == true);
		assert(child6.isLeaf() == true);
		
		assert(child6.getKey(1).getName().equals("Y"));
		assert(child6.getKey(2).getName().equals("Z"));		
	}

	public void validateDeleteC() {

		assert(root.getNumKeys() == 4);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);
		child3 = root.getChild(3);
		child4 = root.getChild(4);
		child5 = root.getChild(5);

		assert(child1.getNumKeys() == 4);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 3);
		assert(child4.getNumKeys() == 2);
		assert(child5.getNumKeys() == 2);

		assert(child1.isLeaf() == true);
		assert(child2.isLeaf() == true);		
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		assert(child5.isLeaf() == true);
		
		assert(child5.getKey(1).getName().equals("Y"));
		assert(child5.getKey(2).getName().equals("Z"));		
	}

	public void validateDeleteP() {

		assert(root.getNumKeys() == 4);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);
		child3 = root.getChild(3);
		child4 = root.getChild(4);
		child5 = root.getChild(5);

		assert(child1.getNumKeys() == 4);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 2);
		assert(child4.getNumKeys() == 2);
		assert(child5.getNumKeys() == 2);

		assert(child1.isLeaf() == true);
		assert(child2.isLeaf() == true);		
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		assert(child5.isLeaf() == true);
		
		assert(child5.getKey(1).getName().equals("Y"));
		assert(child5.getKey(2).getName().equals("Z"));				
	}

	public void validateDeleteV() {

		assert(root.getNumKeys() == 3);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);
		child3 = root.getChild(3);
		child4 = root.getChild(4);

		assert(child1.getNumKeys() == 4);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 4);
		assert(child4.getNumKeys() == 2);

		assert(child1.isLeaf() == true);
		assert(child2.isLeaf() == true);		
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
		
		assert(child4.getKey(1).getName().equals("Y"));
		assert(child4.getKey(2).getName().equals("Z"));				
	}
	

	public void validateDeleteY() {

		assert(root.getNumKeys() == 3);
		assert(root.isLeaf() == false);

		child1 = root.getChild(1);
		child2 = root.getChild(2);
		child3 = root.getChild(3);
		child4 = root.getChild(4);

		assert(child1.getNumKeys() == 4);
		assert(child2.getNumKeys() == 2);
		assert(child3.getNumKeys() == 3);
		assert(child4.getNumKeys() == 2);

		assert(child1.isLeaf() == true);
		assert(child2.isLeaf() == true);		
		assert(child3.isLeaf() == true);
		assert(child4.isLeaf() == true);
	}	
}
