/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util.btree;

public class MemoryStore extends AbstractStore {
	public static final int DEFAULT_BLOCK_SIZE = 8192;

	public MemoryStore(int entrySize, IndexTypeEnum type) {
		super(DEFAULT_BLOCK_SIZE, entrySize, type);
	}	

	public MemoryStore(int blockSize, int entrySize, IndexTypeEnum type) {
		super(blockSize, entrySize, type);
	}

	@Override
	public void save(Node x) {

	}

	@Override
	public void load(Node x) {
		if(x.getAddress() == 0) {
			x.setLeaf(true);
			x.setAddress(System.identityHashCode(x)); // Give it a unique address
		}
		cachedNodes.put(x.getAddress(), x);		
	}

	@Override
	public int getHeaderSize() {
		return 0;
	}

	@Override
	public IndexType getIndexType() {
		if(this.type == IndexTypeEnum.STRING)
			return new StringIndexType(entrySize);

		return null;
	}

	@Override
	public void setRoot(Node root) {
		rootAddress = root.getAddress();		
	}
}