/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util.btree;

public interface Node {
	public static final int NODE_HEADER_LEN = Integer.SIZE/8 + Long.SIZE/8 + Byte.SIZE/8; // header has length of numKeys, offset and leaf datatypes	
	public static final int CHILD_PTR_LEN = Long.SIZE/8; // offset datatype length

	public byte[] getData();

	public long getAddress();

	public void setAddress(long value);
	
	public boolean isLeaf();
	
	public void setLeaf(boolean value);

	/**
	 * Event handler triggered after the data is loaded
	 */
	public void postLoad();

	/**
	 * Event handler triggered before the save is performed giving the chance for the client to initialize fields
	 */
	public void preSave();

	/**
	 * The number of keys currently in the node
	 * @return
	 */
	public int getNumKeys();
	
	/**
	 * Sets the number of keys in the node
	 * @param i
	 */
	public void setNumKeys(int i);	

	/**
	 * Returns the child node as the ith position. The position starts from 1.
	 * @param i
	 * @return
	 */
	public Node getChild(int i);

	/**
	 * Sets the child node at the ith position.
	 * @param i
	 * @param child
	 */
	public void setChild(int i, Node child);	
	
	/**
	 * Get the get at the ith position. The position starts from 1.
	 * @param keyIndex
	 * @return
	 */
	public Key getKey(int i);

	/**
	 * Set the key at the ith position
	 * @param keyIndex
	 * @param key
	 */
	public void setKey(int i, Key key);

	/**
	 * remove the ith key and i+1th child pointer from the node
	 * @param i
	 */	
	public void removeKey(int i);
}